import { Card } from '@/components/ui/card';
import { Bot, MessageSquare, Zap, CornerRightDown, ArrowRight, LucideIcon, Wallet, Search, Clock3 } from 'lucide-react';
import { motion } from 'framer-motion';
import telegramScreenshot from '../assets/telegram-screenshot.jpeg';

const steps = [
  {
    number: 1,
    title: "Connect Wallet",
    description: "Securely connect your BSC wallet to Quikie Bot with a simple command for instant trading access.",
    icon: Wallet
  },
  {
    number: 2,
    title: "Scan a Token",
    description: "Analyze any BSC token with AI-powered scanning to detect potential risks and verify contract safety.",
    icon: Search
  },
  {
    number: 3,
    title: "Trade in <3 Secs",
    description: "Execute lightning-fast trades on Binance Smart Chain with our proprietary speed optimization technology.",
    icon: Clock3
  }
];

// Telegram message interface component
const TelegramMessage = ({ 
  isBot = false, 
  content, 
  time = "11:42 AM"
}: { 
  isBot?: boolean; 
  content: string | React.ReactNode; 
  time?: string;
}) => (
  <div className={`flex ${isBot ? '' : 'justify-end'} mb-4 group`}>
    {isBot && (
      <div className="w-8 h-8 mr-2 flex-shrink-0 rounded-full bg-gradient-to-br from-green-500 to-green-600 flex items-center justify-center text-white">
        <Bot className="h-4 w-4" />
      </div>
    )}
    <div className={`max-w-[75%] rounded-2xl p-3 ${
      isBot 
        ? 'bg-[#1E2740] text-white shadow-md' 
        : 'bg-[#2B5278] text-white shadow-md'
      }`}
    >
      {content}
      <div className="text-xs text-gray-400 text-right mt-1 flex items-center justify-end">
        <span>{time}</span>
        {!isBot && (
          <svg className="h-3 w-3 ml-1 text-blue-400" viewBox="0 0 24 24">
            <path d="M9 16.17L4.83 12l-1.42 1.41L9 19 21 7l-1.41-1.41L9 16.17z" fill="currentColor" />
          </svg>
        )}
      </div>
    </div>
  </div>
);

export default function HowItWorksSection() {
  return (
    <div className="bg-background py-24">
      <div className="max-w-7xl mx-auto px-4 sm:px-6 lg:px-8">
        <div className="text-center mb-16">
          <h2 className="text-4xl md:text-5xl font-bold mb-4">How It Works</h2>
          <p className="text-gray-300 max-w-3xl mx-auto text-lg">
            Start trading with our Telegram-powered AI bot in just three simple steps
          </p>
        </div>
        
        <div className="grid grid-cols-1 lg:grid-cols-2 gap-12 items-center">
          {/* Phone Frame showing Telegram Bot */}
          <div className="order-2 lg:order-1 flex justify-center">
            <div className="relative w-[320px] h-[650px]">
              {/* Phone frame */}
              <div className="absolute inset-0 bg-[#111] rounded-[40px] shadow-2xl overflow-hidden border-8 border-[#222]">
                {/* Status bar */}
                <div className="bg-black h-7 flex items-center justify-between px-6">
                  <div className="text-white text-xs">9:41</div>
                  <div className="flex space-x-2">
                    <div className="w-4 h-2 bg-white rounded-sm"></div>
                    <div className="w-2 h-2 bg-white rounded-full"></div>
                    <div className="text-white text-xs">100%</div>
                  </div>
                </div>
                
                {/* Notch */}
                <div className="absolute top-0 left-1/2 transform -translate-x-1/2 w-40 h-6 bg-black rounded-b-xl z-10"></div>
                
                {/* Telegram app header */}
                <div className="bg-[#232F3E] text-white p-3 flex items-center">
                  <ArrowRight className="h-5 w-5 mr-3 rotate-180" />
                  <div className="flex items-center">
                    <div className="w-10 h-10 rounded-full bg-gradient-to-br from-green-500 to-green-600 flex items-center justify-center text-white font-bold mr-3">
                      <Bot className="h-6 w-6" />
                    </div>
                    <div>
                      <div className="font-medium">Quikie Bot</div>
                      <div className="flex items-center text-xs text-green-300">
                        <span className="h-2 w-2 rounded-full bg-green-400 mr-1"></span>
                        Online
                      </div>
                    </div>
                  </div>
                </div>
                
                {/* Actual Telegram screenshot */}
                <div className="h-[calc(100%-80px)] overflow-hidden">
                  <img 
                    src={telegramScreenshot} 
                    alt="Telegram Bot Screenshot" 
                    className="w-full object-cover object-top"
                    style={{ height: "115%", marginTop: "-40px" }} 
                  />
                </div>
              </div>
              
              {/* Glow effect behind phone */}
              <div className="absolute -inset-4 bg-gradient-to-br from-blue-500/20 to-green-500/20 rounded-[50px] blur-xl -z-10"></div>
            </div>
          </div>
          
          {/* Steps */}
          <div className="order-1 lg:order-2 flex flex-col justify-center space-y-8">
            {steps.map((step, index) => (
              <motion.div 
                key={step.number}
                initial={{ opacity: 0, x: 50 }}
                whileInView={{ opacity: 1, x: 0 }}
                transition={{ delay: index * 0.2 }}
                viewport={{ once: true }}
              >
                <Card className="bg-secondary/30 backdrop-blur-sm border border-accent/10 p-6 flex items-start relative overflow-hidden">
                  {/* Background glow */}
                  <div className="absolute top-0 right-0 w-32 h-32 bg-accent/5 rounded-full blur-2xl transform translate-x-1/2 -translate-y-1/2"></div>
                  
                  <div className="bg-accent/10 h-12 w-12 rounded-full flex items-center justify-center flex-shrink-0 mr-4 relative">
                    <step.icon className="h-6 w-6 text-accent" />
                    <div className="absolute -top-1 -right-1 bg-accent h-5 w-5 rounded-full flex items-center justify-center text-xs font-bold">
                      {step.number}
                    </div>
                  </div>
                  <div>
                    <h3 className="font-bold text-lg mb-2">{step.title}</h3>
                    <p className="text-gray-300">{step.description}</p>
                  </div>
                </Card>
              </motion.div>
            ))}
            
            <motion.div
              initial={{ opacity: 0, y: 20 }}
              whileInView={{ opacity: 1, y: 0 }}
              transition={{ delay: 0.6 }}
              viewport={{ once: true }}
              className="mt-6"
            >
              <a 
                href="https://t.me/SuperQuikieBot" 
                target="_blank" 
                rel="noopener noreferrer"
                className="inline-flex items-center text-accent font-medium"
              >
                <span>Learn more about our Telegram bot</span>
                <ArrowRight className="ml-2 h-4 w-4" />
              </a>
            </motion.div>
          </div>
        </div>
      </div>
    </div>
  );
}
