import { motion, useAnimation } from 'framer-motion';
import { useState, useEffect, useRef } from 'react';
import { ChevronLeft, ChevronRight } from 'lucide-react';

const partners = [
  {
    name: "PancakeSwap",
    logo: "https://www.pngall.com/wp-content/uploads/10/PancakeSwap-Crypto-Logo-PNG.png",
    color: "#1FC7D4"
  },
  {
    name: "4Meme",
    logo: "https://cdn.galxe.com/galaxy/o89dunygdvyusp9dys3dfk/1747f4c6-ddfe-43aa-9dac-6ab84fdfd0fa.jpeg",
    color: "#F063A9"
  },
  {
    name: "BabyDoge",
    logo: "https://th.bing.com/th/id/OIP.Tb01HkZFaDyDLB1F2nvN_gHaFj?rs=1&pid=ImgDetMain",
    color: "#FFD54F"
  },
  {
    name: "Binance",
    logo: "https://public.bnbstatic.com/static/images/common/favicon.ico",
    color: "#F3BA2F"
  },
  {
    name: "Coinbase",
    logo: "https://www.clker.com/cliparts/3/5/6/8/1652157600960257271coinbase-logo-square-1.png",
    color: "#0052FF"
  }
];

export default function PartnersSection() {
  const [active, setActive] = useState(0);
  const [direction, setDirection] = useState(0);
  const containerRef = useRef<HTMLDivElement>(null);
  const controls = useAnimation();
  
  const getVisibleItemCount = () => {
    if (typeof window !== 'undefined') {
      if (window.innerWidth < 640) return 2;
      if (window.innerWidth < 1024) return 3;
      return 4;
    }
    return 4;
  };
  
  const [visibleItems, setVisibleItems] = useState(getVisibleItemCount());
  
  useEffect(() => {
    const handleResize = () => {
      setVisibleItems(getVisibleItemCount());
    };
    
    window.addEventListener('resize', handleResize);
    handleResize();
    
    return () => window.removeEventListener('resize', handleResize);
  }, []);
  
  // Auto-scroll every 3 seconds
  useEffect(() => {
    const interval = setInterval(() => {
      handleNext();
    }, 3000);
    
    return () => clearInterval(interval);
  }, [active]);
  
  const handlePrev = () => {
    if (active <= 0) return;
    setDirection(-1);
    setActive(prev => prev - 1);
  };
  
  const handleNext = () => {
    if (active >= partners.length - visibleItems) {
      setDirection(-1);
      setActive(0);
      return;
    }
    setDirection(1);
    setActive(prev => prev + 1);
  };
  
  const variants = {
    enter: (direction: number) => {
      return {
        x: direction > 0 ? 1000 : -1000,
        opacity: 0
      };
    },
    center: {
      zIndex: 1,
      x: 0,
      opacity: 1
    },
    exit: (direction: number) => {
      return {
        zIndex: 0,
        x: direction < 0 ? 1000 : -1000,
        opacity: 0
      };
    }
  };
  
  return (
    <div className="bg-secondary py-16">
      <div className="max-w-7xl mx-auto px-4 sm:px-6 lg:px-8">
        <div className="text-center mb-12">
          <h2 className="text-4xl md:text-5xl font-bold mb-4">Industry Partners</h2>
          <p className="text-gray-300 max-w-3xl mx-auto text-lg">
            We integrate with leading BSC platforms to provide you with a seamless trading experience on Binance Smart Chain.
          </p>
        </div>
        
        <div className="relative">
          {/* Carousel Navigation */}
          <div className="absolute -left-4 top-1/2 transform -translate-y-1/2 z-10">
            <button 
              onClick={handlePrev}
              disabled={active <= 0}
              className={`rounded-full p-2 bg-background/80 hover:bg-background border border-accent/20 ${active <= 0 ? 'opacity-50 cursor-not-allowed' : 'cursor-pointer'}`}
              aria-label="Previous"
            >
              <ChevronLeft className="h-6 w-6 text-accent" />
            </button>
          </div>
          
          <div className="absolute -right-4 top-1/2 transform -translate-y-1/2 z-10">
            <button 
              onClick={handleNext}
              disabled={active >= partners.length - visibleItems}
              className={`rounded-full p-2 bg-background/80 hover:bg-background border border-accent/20 ${active >= partners.length - visibleItems ? 'opacity-50 cursor-not-allowed' : 'cursor-pointer'}`}
              aria-label="Next"
            >
              <ChevronRight className="h-6 w-6 text-accent" />
            </button>
          </div>
          
          {/* Partners Display */}
          <div 
            ref={containerRef} 
            className="overflow-hidden py-8 px-4"
          >
            <div 
              className="flex transition-transform duration-500 ease-out"
              style={{ transform: `translateX(-${active * (100 / visibleItems)}%)` }}
            >
              {partners.map((partner, index) => (
                <motion.div 
                  key={partner.name}
                  className="text-center group flex-none px-4"
                  style={{ width: `${100 / visibleItems}%` }}
                  initial={{ opacity: 0.6 }}
                  animate={{ opacity: 1 }}
                  whileHover={{ 
                    y: -5,
                    transition: { duration: 0.2 } 
                  }}
                  whileTap={{ scale: 0.95 }}
                >
                  <div className="h-24 w-24 mx-auto flex items-center justify-center relative overflow-hidden rounded-lg">
                    <div className="absolute inset-0 bg-gray-800 filter blur opacity-40 group-hover:opacity-60 transition-opacity" />
                    <div className="absolute inset-0 bg-gradient-to-br from-transparent to-white/10 opacity-0 group-hover:opacity-50 transition-opacity" />
                    <img 
                      src={partner.logo} 
                      alt={partner.name}
                      className="h-16 w-16 object-contain relative z-10 transition-all duration-300 group-hover:scale-110" 
                    />
                  </div>
                  <div className="mt-3 font-medium text-gray-300 group-hover:text-white transition-colors">
                    {partner.name}
                  </div>
                </motion.div>
              ))}
            </div>
          </div>
          
          {/* Dots Indicator */}
          <div className="flex justify-center mt-4 space-x-2">
            {Array.from({ length: partners.length - visibleItems + 1 }).map((_, index) => (
              <button
                key={index}
                onClick={() => {
                  setDirection(index > active ? 1 : -1);
                  setActive(index);
                }}
                className={`h-2 w-2 rounded-full ${index === active ? 'bg-accent' : 'bg-gray-600'}`}
                aria-label={`Go to slide ${index + 1}`}
              />
            ))}
          </div>
        </div>
      </div>
    </div>
  );
}
